/*!  \file 

$Id: kernel.h,v 1.1 2003/04/5 18:22:25 tsp_admin Exp $

-----------------------------------------------------------------------

GDISP+ - Graphic Tool based on GTK+,
         for being used upon the generic Transport Sampling Protocol.

Copyright (c) 2003 - Yves DUFRENNE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-----------------------------------------------------------------------

Project   : TSP
Maintainer: tsp@astrium-space.com
Component : Graphic Tool

-----------------------------------------------------------------------

Purpose   : Graphic Tool based on GTK+ that provide several kinds of
            plot. This tool is to be used with the generic TSP protocol.

File      : Graphic Tool Kernel Interface.
            Definition of all structures and types for the application.

-----------------------------------------------------------------------
*/

#ifndef __KERNEL_H__
#define __KERNEL_H__

#include <gtk/gtk.h>


/*
 * Time management.
 * Gestion du temps "haute resolution".
 * Voir "utils.c" pour la compilation autre que linux.
 */
typedef gint64 HRTime_T;


/*
 * Colors.
 * Sur l'ensemble des 170 couleurs que compte la colormap que j'ai
 * creee, voila un petit ensemble qui suffit pour dessiner ce que
 * vous voulez.
 * Pour utiliser une couleur, il suffit de prendre sa definition
 * (autrement appelee 'pixel') dans le kernel de la facon suivante :
 *
 *       GtkColor *myBlackColor = &kernel->colors[_BLACK_];
 *
 * Voir "plot2D.c" pour une utilisation des couleurs.
 *
 * ATTENTION : Sur vos stations 8 plans (donc 256 couleurs maximum),
 * de type 'breiz', le fait que je cre autant de couleurs posera
 * peut-etre des problemes. Donc, essayez d'avoir une minimum de clients
 * X a l'ecran. A terme, je n'imposerai pas autant de couleurs.
 */
#define _RED_       3
#define _GREEN_    10
#define _YELLOW_   17
#define _BLUE_     24
#define _MAGENTA_  31
#define _CYAN_     38
#define _BLACK_    42
#define _GREY_    155
#define _WHITE_   169


/*
 * Font management.
 * Vous disposez de 9 fontes differentes pour ecrire dans une zone graphique.
 * Les 9 fontes sont obtenues grace au couple (fontType, fontStyle).
 * Pour utiliser une fonte, on pioche encore dans le kernel :
 *
 *   GtkFont *myFont = kernel->fonts[GD_FONT_BIG][GD_FONT_FIXED];
 *
 * Voir "plot2D.c" pour une utilisation des fontes.
 *
 * ATTENTION au positionnement des chaines de catacteres.
 * Si vous ecrivez en (x,y), ce n'est pas le coin superieur gauche du rectangle
 * qui englobe la chaine qui sera a cette position. C'est assez delicat.
 * Je rectifierai si vous avez des problemes.
 */
typedef enum {

  GD_FONT_NORMAL = 0,
  GD_FONT_ITALIC,
  GD_FONT_FIXED,
  GD_FONT_MAX_TYPE

} FontType_T;

typedef enum {

  GD_FONT_SMALL = 0,
  GD_FONT_MEDIUM,
  GD_FONT_BIG,
  GD_FONT_MAX_SIZE

} FontSize_T;


/*
 * Kernel alias. No comment.
 */
typedef struct Kernel_T_ *Kernel_T_Ptr;


/*
 * Definition of the operating system of a given typed plot.
 *
 * Voila maintenant comment je definis un plot graphique.
 * Dans "plot2D.c", toutes les procedures/fonctions implementees sont "static",
 * mise a part la derniere, qui est la seule exportee.
 *
 * Voila donc une structure (toujours stockee dans le kernel) qui offre tout
 * ce qu'il faut pour creer/destruire/attacher/montrer/dimensionner un plot
 * graphique.
 *
 * Donc le code dans "page.c" se contente d'appeler ces fonctions-la, qui dans
 * la vraie implentation de gdisp+ sont initialisees en fonction du tyoe de
 * plot que l'on souhaite creer.
 * Mais ici, un seul type de plot est gere, le plot 2D.
 *
 */
typedef struct PlotSystem_T_ {

  gboolean psIsSupported;

  /*
   * ATTENTION : l'ordre suivant de presentation des fonctions n'est PAS
   * l'ordre reel d'appel lors de l'execution.
   * Voir "page.c" pour l'ordre d'exact d'appel.
   */

  /*
   * Fonction de creation des donnees privees du plot graphique.
   * C'est dans cette fonction que l'on alloue dynamiquement (PAS DE 
   * VARIABLES GLOBALES) une structure definie dans "plot2D.h" et que
   * l'on effectue quelques initialisations.
   * L'adresse de cette structure est retournee au process appelant sous
   * forme d'un type opaque (void*).
   *
   * Dans toutes les autres procedures/fonctions, le deuxieme parametre,
   * apres le pointeur sur le kernel, est justement la structure privee
   * allouee, ce qui permet a chaque instance de plot de retrouver ses petits.
   */
  void      *(*psCreate )          (Kernel_T_Ptr                         );

  /*
   * Destruction des donnees privees du plot graphique. No comment.
   */
  void       (*psDestroy)          (Kernel_T_Ptr,void*                   );

  /*
   * Permet au plot 2D d'enregistrer son widet GTK parent.
   * Pour l'instant, cette procedure existe pour un besoin futur.
   * Le parent enregistre n'est pas utilise pour l'instant.
   *
   * De plus, ce n'est pas au plot lui-meme de s'attacher a son parent
   * car il en est incapable. En effet, le plot de connait pas le "type"
   * de "container" qui lui sert de parent. On ne s'attache pas a une
   * "fenetre graphique" de la meme facon que l'on s'attache a une "hbox"
   * ou a une "ctable", qui sont divers "containers" GTK.
   */
  void       (*psSetParent)        (Kernel_T_Ptr,void*,GtkWidget*        );

  /*
   * Par contre, le plot graphique se doit de remonter a l'appelant son
   * widget de plus haut niveau (forcement un 'container' au sens GTK) de
   * facon a ce qu'il soit attache a la page graphique creee.
   * C'est bien a la page d'attacher ses divers fils graphiques.
   */
  GtkWidget *(*psGetTopLevelWidget)(Kernel_T_Ptr,void*                   );

  /*
   * Il faut bien montrer le plot graphique a un moment...
   */
  void       (*psShow)             (Kernel_T_Ptr,void*                   );

  /*
   * Il faut bien specifier les dimensions INITIALES a un moment...
   */
  void       (*psSetDimensions)    (Kernel_T_Ptr,void*,guint,guint       );

  /*
   * Installation et recuperation des symboles.
   * PAS UTILE dans la version standalone de gdisp+.
   */
  void       (*psAddSymbols)       (Kernel_T_Ptr,void*,GList*,guint,guint);
  GList     *(*psGetSymbols)       (Kernel_T_Ptr,void*,gchar             );

  /*
   * C'est la methode dans laquelle vous faites tout ce que vous voulez
   * pour rafraichier le contenu de l'espace graphique du plot 2D.
   *
   * Je n'ai pas l'intention d'ajouter de parametres a ce methode (comme
   * par exemple la liste des dernieres valeurs des symboles). C'est par
   * l'intermediaire des symboles installes (deux methodes precedentes)
   * que j'accederai aux valeurs (le symboles est un objet complet).
   */
  void       (*psStep)             (Kernel_T_Ptr,void*                   );

} PlotSystem_T;


/*
 * Kernel structure.
 */
typedef struct Kernel_T_ {

  /*
   * Font, Colormap and color management.
   */
  GdkColormap       *colormap;
  GdkVisual         *visual;
  GdkColor          *colors;
  gint               colorNumber;
  GdkFont           *fonts[GD_FONT_MAX_SIZE][GD_FONT_MAX_TYPE];

  /*
   * Definition of the 2D plot.
   */
  GtkWidget         *pageWindow; /* La "topLevelWindow" de la page   */

  PlotSystem_T       plotSystem; /* le fameux plot system...         */
  void              *plotData;   /* ... et sa donnee privee associee */

  gint               gdispTimeout;
  guint              timerPeriod; /* milli-seconds                   */
  guint              timerWatchDog;

} Kernel_T;


#endif /* __KERNEL_H__ */
