/*!  \file 

$Id: page.c,v 1.1 2003/04/5 18:22:25 tsp_admin Exp $

-----------------------------------------------------------------------

GDISP+ - Graphic Tool based on GTK+,
         for being used upon the generic Transport Sampling Protocol.

Copyright (c) 2003 - Yves DUFRENNE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

-----------------------------------------------------------------------

Project   : TSP
Maintainer: tsp@astrium-space.com
Component : Graphic Tool

-----------------------------------------------------------------------

Purpose   : Graphic Tool based on GTK+ that provide several kinds of
            plot. This tool is to be used with the generic TSP protocol.

File      : Graphic page management.

-----------------------------------------------------------------------
*/

#ifndef __PAGES_C__
#define __PAGES_C__


/*
 * System includes.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>


/*
 * GDISP+ includes.
 */
#include "kernel.h"
#include "prototypes.h"


/*
 * Page border width.
 */
#define GD_PAGE_BORDER_WIDTH  5


/*
 --------------------------------------------------------------------
                             STATIC ROUTINES
 --------------------------------------------------------------------
*/


/*
 * The "delete_event" occurs when the window manager sens this event
 * to the application, usually by the "close" option, or on the titlebar.
 * Returning TRUE means that we do not want to have the "destroy" event 
 * emitted, keeping GDISP+ running. Returning FALSE, we ask that "destroy"
 * be emitted, which in turn will call the "destroy" signal handler.
 */
static gint
gdispManageDeleteEventFromWM (GtkWidget *pageWindow,
			      GdkEvent  *event,
			      gpointer   data)
{

  return FALSE;

}


/*
 * The "destroy" event occurs when we call "gtk_widget_destroy" on
 * the top-level window, of if we return FALSE in the "delete_event"
 * callback (see above).
 */
static void
gdispDestroySignalHandler (GtkWidget *pageWindow,
			   gpointer   data)
{

  Kernel_T *kernel = (Kernel_T*)data;

  /*
   * Destroy the plot.
   */
  if (kernel->gdispTimeout > 0) {

    gtk_timeout_remove(kernel->gdispTimeout);
    kernel->gdispTimeout = 0;

  }

  (*kernel->plotSystem.psDestroy)(kernel,
				  kernel->plotData);

  kernel->plotData = (void*)NULL;

  gtk_widget_destroy(kernel->pageWindow);
  gtk_main_quit();

}


/*
 --------------------------------------------------------------------
                             PUBLIC ROUTINES
 --------------------------------------------------------------------
*/


/*
 * Create GDISP+ graphic page.
 */
void
gdispCreateGraphicPage ( Kernel_T *kernel )
{

  GdkGeometry  pWindowHints;
  GtkWidget   *plotTopLevel = (GtkWidget*)NULL;
  gint         screenWidth  =                0;
  gint         screenHeight =                0;


  /* ------------------------ MAIN WINDOW ------------------------ */


  /*
   * Now, deal with graphic page top-level window.
   * The GTK_WINDOW_TOPLEVEL argument specifies that we want the window
   * to undergo window manager decoration and placement.
   */
  screenWidth  = gdk_screen_width ();
  screenHeight = gdk_screen_height();

  kernel->pageWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  assert(kernel->pageWindow);

  gtk_signal_connect(GTK_OBJECT(kernel->pageWindow),
		     "delete_event",
		     GTK_SIGNAL_FUNC(gdispManageDeleteEventFromWM),
		     (gpointer)kernel);

  gtk_signal_connect(GTK_OBJECT(kernel->pageWindow),
		     "destroy",
		     GTK_SIGNAL_FUNC(gdispDestroySignalHandler),
		     (gpointer)kernel);


  /*
   * Set up window size, title and border width.
   */
  gtk_widget_set_usize(GTK_WIDGET(kernel->pageWindow),
		       screenWidth  / 2,  /* width  */
		       screenHeight / 2); /* height */

  gtk_window_set_title(GTK_WINDOW(kernel->pageWindow),
		       "GDISP+ Graphic Page.");

  gtk_container_set_border_width(GTK_CONTAINER(kernel->pageWindow),
				 GD_PAGE_BORDER_WIDTH);


  /*
   * Tell the window manager to maintain the original aspect ratio.
   */
  pWindowHints.min_aspect = (gdouble)screenWidth / (gdouble)screenHeight;
  pWindowHints.max_aspect = pWindowHints.min_aspect;

  gtk_widget_show(kernel->pageWindow);

  gdk_window_set_geometry_hints(GTK_WIDGET(kernel->pageWindow)->window,
				&pWindowHints,
				GDK_HINT_ASPECT);


  /* ************************************************************
   *
   * C'est a partir d'ici que commence la creation du plot 2D.
   *
   * ************************************************************/

  /*
   * Create the 2D plot.
   */
  kernel->plotData = (*kernel->plotSystem.psCreate)(kernel);


  /*
   * Set parent widget.
   */
  (*kernel->plotSystem.psSetParent)(kernel,
				    kernel->plotData,
				    kernel->pageWindow);

  /*
   * Set initial dimensions.
   */
  (*kernel->plotSystem.psSetDimensions)(kernel,
					kernel->plotData,
					screenWidth  / 2,
					screenHeight / 2);

   /*
    * Attach new plot to its parent.
    */
  plotTopLevel = (*kernel->plotSystem.psGetTopLevelWidget)(kernel,
							   kernel->plotData);
  gtk_container_add(GTK_CONTAINER(kernel->pageWindow),
		    plotTopLevel);


  /*
   * Show the plot.
   *
   * ATTENTION : normalement, dans une application X, tant que l'on a pas
   * rendu la main a la "main loop X11", aucun callback attache aux evenements
   * X (expose, configure, ...etc...) n'est appele.
   * Il se trouve que lors de l'execution de "psShow", le callback associe
   * a l'evenement "configureNotify" est appele. Je ne sais pas pourquoi.
   *
   * Tout ca pour vous dire que l'appel a la methode qui suit va vous faire
   * passer par "gdisp_showPlot2D", ce qui est normal, mais aussi par
   * "gdisp_plot2DConfigure", ce qui est moins normal pour moi, surtout
   * quand on voir les valeurs des dimensions proposees.
   *
   * Mais on fait avec. Voir le debut de cette methode.
   */
  (*kernel->plotSystem.psShow)(kernel,
			       kernel->plotData);

  /* ************************************************************
   *
   * Le plot graphique est maintenant cree.
   *
   * ************************************************************/

}

#undef BENCHMARK

gint
gdispPageStep ( void *data )
{

  Kernel_T        *kernel    = (Kernel_T*)data;

#if defined(BENCHMARK)

  HRTime_T         startMark = (HRTime_T)0;
  HRTime_T         stopMark  = (HRTime_T)0;
  HRTime_T         deltaMark = (HRTime_T)0;
  static HRTime_T  lastMark  = (HRTime_T)0;

#endif

  /*
   * Wait for one second before really starting the graphic process.
   * MERCI DE ME CONTACTER POUR SAVOIR POURQUOI JE FAIS CA.
   * IL EST TARD, J'AI LA FLEME D'ECRIRE ET J'AI MAL AU BASSIN.
   */
  if (kernel->timerWatchDog > 0) {
    kernel->timerWatchDog--;
    return TRUE;
  }

  /*
   * Check GTK timer.
   * ESSAYEZ, vous verrez que ce n'est pas tres precis, et c'est normal.
   * On n'est pas en temps reel...
   *
   * La periode d'activation est fixee dans "gdisp.c" (ligne 98).
   */
#if defined(BENCHMARK)

  if (lastMark > 0) {

    deltaMark = gdisp_getHRTime() - lastMark;

    fprintf(stdout,
	    "GTK Timer : %d nanoseconds\n",
	    (int)(deltaMark & 0xFFFFFFFF));

  }

#endif

  /*
   * Do the plot step.
   */
#if defined(BENCHMARK)

  startMark = gdisp_getHRTime();

#endif

  /* ************************************************************
   *
   * Le plus interessant est ici.
   *
   * ************************************************************/

  (*kernel->plotSystem.psStep)(kernel,
			       kernel->plotData);

  /* ************************************************************
   *
   * Le plus interessant est ici.
   *
   * ************************************************************/

#if defined(BENCHMARK)

  stopMark = gdisp_getHRTime();

  deltaMark = stopMark - startMark;

  fprintf(stdout,
	  "Plot 2D   : %d nanoseconds\n",
          (int)(deltaMark & 0xFFFFFFFF));

  lastMark = gdisp_getHRTime();

#endif

  return TRUE;

}

#endif /* __PAGES_C__ */
